; -----------------------------------------------------------------------------
; Display intro before game starts.
; -----------------------------------------------------------------------------
; Coded by Einar Saukas
; -----------------------------------------------------------------------------

        org     44707

; -----------------------------------------------------------------------------
; SoundTracker player and music "AI"
; -----------------------------------------------------------------------------
stracker_ai:
INCBIN  "STPLAYER_AI_44707.bin"

; -----------------------------------------------------------------------------
; Music "Chouyaku"
; -----------------------------------------------------------------------------
chouyaku:
INCBIN  "CHOUYAKU_49202.bin"

; -----------------------------------------------------------------------------
; Control intro, i.e. play music "Prelude" at loading screen, wait for key,
; clear screen, show intro text, and decompress BIFROST*.
; -----------------------------------------------------------------------------
intro_game:
        xor     a
        ld      (23560), a              ; reset last pressed key
        call    $229b                   ; BORDER 0

        ld      hl, prelude
        ld      (stracker_ai+1), hl     ; select music "prelude"
        call    stracker_ai             ; enable Soundtracker Player
intro_loop:
        halt                            ; wait for next frame
        call    stracker_ai+6           ; play next tick from Soundtracker Player
        ld      a, (23560)
        and     a
        jr      z, intro_loop           ; repeat until a key is pressed

        call    stracker_ai             ; disable Soundtracker Player

        ld      hl, fzx_typewriter
        ld      (FZX_DRIVER+60), hl     ; configure font "typewriter.fzx"

        call    intro_print             ; print intro text

        ld      hl, multi_render
        ld      de, $ded7
        call    dzx7_agilercs           ; decompress BIFROST* with customized render

        call    wait_new_key

        ld      hl, interrupt_handler   ; interrupt_handler
        ld      ($fdfe), hl             ; configure BIFROST* start to call interrupt_handler directly
        ld      (64993), hl             ; configure BIFROST* render to call interrupt_handler afterwards

        ret

; -----------------------------------------------------------------------------
; Display intro text.
; -----------------------------------------------------------------------------
intro_print:
        ld      a, 6
        ld      (23693), a              ; INK 6; PAPER 0; BRIGHT 0; FLASH 0
        call    $0daf                   ; CLS

        ld      hl, intro_text
        xor     a
        ld      (23560), a              ; reset last pressed key
intro_next_slower:
        halt
        halt                            ; short delay
intro_next_faster:
        ld      a, (hl)                 ; read next char from text
        inc     hl
        cp      255
        ret     z                       ; exit if 255 (end of message)
        push    hl
        call    FZX_START               ; print char
        pop     hl
        ld      a, (23560)
        and     a
        jr      nz, intro_next_faster   ; print next char immediately if player pressed any key
        ld      a, (hl)
        cp      33
        jr      c, intro_next_slower    ; skip typewriter sound for non-visible chars
        push    hl
        ld      hl, $00c8               ; beeper pitch
        ld      de, $0004               ; beeper duration
        call    $03b5                   ; play beeper
        pop     hl
        jr      intro_next_slower

; -----------------------------------------------------------------------------
; Intro text message
; -----------------------------------------------------------------------------
intro_text:
        db      22, 1, 2, 35
        db      "And I foresee a mighty hero shall be", 13
        db      "revealed, trained in the ways of ancient", 13
        db      "technologies, whose conscience shall", 13
        db      "encompass all living creatures, and whose", 13
        db      "mind shall seek and defeat evilness at", 13
        db      "every level, from overwhelming giant", 13
        db      "monsters from outer space to the lowest", 13
        db      "rotten apple or disease-infected fly, for", 13
        db      "sometimes a spectacularly glorious feat", 13
        db      "may not be as hard as it seems, and the", 13
        db      "toughest challenges may hide in the most", 13
        db      "insignificant conflicts against evil,", 13
        db      "although no task shall be too big or too", 13
        db      "small for such a true hero, who shall be", 13
        db      "known as the Evil Hunter, whose guidance", 13
        db      "will forever change the balance of the", 13
        db      "universe...  Hey you! Stop recording", 13
        db      "everything I say! Get off my lawn!", 34
        db      22, 172, 20, "/ words of wisdom from THE BOOK OF"
        db      22, 181, 186,"PROPHECY"
        db      255

; -----------------------------------------------------------------------------
; FZX driver, necessary for printing intro text
; -----------------------------------------------------------------------------
INCLUDE "FZXdriver.asm"

; -----------------------------------------------------------------------------
; Font
; -----------------------------------------------------------------------------
fzx_typewriter:
INCBIN  "typewriter.fzx"

; -----------------------------------------------------------------------------

        org     59536

; -----------------------------------------------------------------------------
; Music "Prelude"
; -----------------------------------------------------------------------------
prelude:
INCBIN  "PRELUDE_59536.bin"

; -----------------------------------------------------------------------------
; Compressed BIFROST* with custom multicolor routines
; -----------------------------------------------------------------------------
multi_render:
INCBIN  "mcolor.bin.zx7"

; -----------------------------------------------------------------------------
; Set border color and clear screen.
;
; Parameters:
;   A: BORDER color
;
; Destroys:
;   AF, DE
; -----------------------------------------------------------------------------
cls_scr:
        halt
        out     (254), a                ; set BORDER
        ld      de, 23295
        xor     a
cls_loop:
        ld      (de), a
        dec     de
        bit     3, d
        jr      nz, cls_loop            ; clear screen sttributes
        ret

; -----------------------------------------------------------------------------
; Set border, clear screen, then decompress screen.
;
; Parameters:
;   A: BORDER color
;   HL: Address of compressed screen
;
; Destroys:
;   AF, BC, DE, HL, AF'
; -----------------------------------------------------------------------------
dzx7_scr:
        call    cls_scr
        ld      de, 16384               ; set target address to screen

; -----------------------------------------------------------------------------
; RCS+ZX7 "agile" decompressor
;
; Parameters:
;   HL: Address of compressed data
;   HL: Address of destination data
;
; Destroys:
;   AF, BC, DE, HL, AF'
; -----------------------------------------------------------------------------
INCLUDE "dzx7_agileRCS.asm"             ; decompress

; -----------------------------------------------------------------------------
